@@ -56,11 +56,10 @@
   need _XOPEN_SOURCE defined for pread.
 */
 #if defined(HAVE_PREAD) && !defined(SOLARIS) && !defined(__OpenBSD__) && !defined(__NetBSD__) && !defined(__FreeBSD__) && !defined(TRU64)
-#ifdef _XOPEN_SOURCE
-#undef _XOPEN_SOURCE
-#endif
+#ifndef _XOPEN_SOURCE
 #define _XOPEN_SOURCE 500
 #endif
+#endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -78,6 +77,7 @@
 #endif
 
 #include <sys/mman.h>
+
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
@@ -232,8 +232,8 @@ typedef u_int32_t cnid_t;
  */
 
 struct ad_entry {
-    u_int32_t   ade_off;
-    u_int32_t   ade_len;
+    off_t   ade_off;
+    ssize_t   ade_len;
 };
 
 typedef struct adf_lock_t {
@@ -335,8 +335,12 @@ struct adouble_fops {
 /* synchronization locks */
 #define AD_FILELOCK_BASE (0x80000000)
 #else
+#if _FILE_OFFSET_BITS == 64
+#define AD_FILELOCK_BASE (0x7FFFFFFFFFFFFFFFULL - 9)
+#else
 #define AD_FILELOCK_BASE (0x7FFFFFFF -9)
 #endif
+#endif
 
 /* FIXME:
  * AD_FILELOCK_BASE case
